@@ -5,27 +5,29 @@
 
   // Functionality related to talks
   // Object used to store various data
-  var VestTalk = {
-    json_messages: {
-      data: []
-    }
-  };
+  var VestTalk = { messages:[] };
+  var user = window.user;
   $(function($) {
     $('#contacts').sidebar('attach events', '#contacts_button');
     $('#talks').sidebar('attach events', '#talks_button');
 
     // Fill in #messages with the last talk and
     // bind onclick to filling-in messages in the #messages
-    $('#talks ul li a').click(get_messages);
+    $('#talks ul li a, #contacts ul.contacts li a.last_talk').click(get_messages);
 
     // List messages from the last talk.
     $('#talks ul li a:first').click();
 
-    $(window).blur(stop_polling);
-    $(window).focus(start_polling);
+    $(window).blur(function () {
+      stop_polling();
+      delay_polling(15, 5, 5);
+    });
+    $(window).focus(function () {
+      start_polling();
+    });
 
     //bind click on icons in the contacts sidebar
-    $('#contacts .list .item .comment.outline.icon').click(new_talk);
+    $('#contacts .list .item .comment.outline.link.icon').click(new_talk);
 
     // To send or not to send the message?
     $('#message_form').submit(validate_and_send_message);
@@ -36,11 +38,11 @@
       }
     });
     //behavior for contact form
-    $('#contacts form').submit(function(){return false});
+    $('#contacts form').submit(function(){return false;});
     $('#contacts form .prompt').keydown(function(e){
-      if ( e.altKey ){return false}
+      if ( e.altKey ){return false;}
       if ( this.value.length > 2 ){
-        find_contacts(e)
+        find_contacts(e);
       }
     });
   }); // end $(document).ready(function($)
@@ -50,36 +52,38 @@
    * Performs a GET request to the url found in the 'href' attribute
    * of a talk item and invokes list_messages_from_json() to populate the
    * #messages box.
-   * @return bool false - to prevent the default behaviour of the 'a' tag.
+   * @return void.
    */
-  function get_messages(e) {
-    var link = e.target;
-    // close the sidebar
-    $('#talks').sidebar('hide');
+  function get_messages(e) {/* jshint validthis: true */
+    e.preventDefault();
+
+    // close the sidebars
+    $('#talks,#contacts').sidebar('hide');
 
     // get the messages
-    $.get(link.href, list_messages_from_json);
-    return false;
+    $.get(this.href, list_messages_from_json, 'json');
   }
 
   /**
    * Populates in #mesages list box with the messages found in
-   * the received json.
+   * the received json. Invokes set_seen() and refresh_talks()
+   * @return void
    */
-  function list_messages_from_json(json_messages) {
+  function list_messages_from_json(JSON) {
     var messages = $('#messages .ui.list');
     messages.html('');
     //Save it for later use by validate_and_send_message
-    VestTalk.json_messages = json_messages;
+    VestTalk.messages = JSON.data;
     var prev_msg = {};
-    $(json_messages.data).each(function(i, msg) {
-
+    var unseen = [];
+    $(VestTalk.messages).each(function(i, msg) {
+      // collect yet unseen messages sent to the current user.
+      if(user.id === msg.to_uid && (msg.seen === null || msg.seen === 0)) { unseen.push(msg.id); }
       // This is the message defining the topic (the parent message).
       if (msg.subject_message_id === 0) {
         set_talk_form(msg); // Set the topic
         //append the first message if there is no offset
-        if (json_messages.links[0].href.match('offset=[1-9]'))
-          return;
+        if (JSON.links[0].href.match('offset=[1-9]')) { return; }
       }
 
       // fill in template and display the message
@@ -89,19 +93,46 @@
     });
     // Scroll down to the last message
     messages.parent().scrollTop(messages.height());
+    set_seen((VestTalk.messages[1] ? VestTalk.messages[1].subject_message_id : 0 ), unseen);
+    refresh_talks(JSON.meta.talks);
   } // end function list_talk_messages_from_json
 
   /**
    * Stop polling for new messages from previous talk.
    */
   function stop_polling() {
-    if (window.new_messages_interval_id) {
+    if (window.new_messages_interval_id > 0) {
       window.clearInterval(window.new_messages_interval_id);
+      window.new_messages_interval_id = 0;
     }
   }
 
+  /**
+   * Starts to call 'get_new_messages'
+   * with an increasing delay of 5 seconds more each next time.
+   * Before each window.setTimeout  window.new_messages_interval_id
+   * will be checked and the delayed execution will be interrupted if
+   * it is greater than 0.
+   * @param times {int} Times new messages will be get
+   * @param interval {int} The first delay in seconds.
+   * @param delay {int} seconds added each next time to the interval.
+   * Example:
+   * start_delayed_polling (5, 5, 5)
+   * 'get_new_messages' will be executed after 10, 15, 20, 25, 30
+  */
+  function delay_polling (times, interval, delay) {
+    //console.log('start_delayed_polling:',(new Date()).getSeconds()); 
+    //console.log('times, interval, delay:',times, interval, delay); 
+    if ((window.new_messages_interval_id > 0) || (times===0)) {return;}
+    window.setTimeout(function () {
+      get_new_messages($('#message_form').get(0),10);
+      delay_polling(times - 1, interval += delay, delay);
+    }, interval * 1000);
+  }
+
   /**
    * Start polling for new messages from the buddy
+   * @return {void}
    */
   function start_polling() {
     stop_polling();
@@ -112,6 +143,24 @@
       }, 5000);
   }
 
+  /**
+   * Sets messages from the messages list as 'seen=1'.
+   * Performs a PUT request with parameters 
+   * 'subject_message_id'(int) and 'unseen' (comma separated string of message IDs).
+   * @param subject_message_id {int} id of the talk
+   * @param unseen {Array} arry of message IDs.
+   * @return void
+   */
+  function set_seen (subject_message_id, unseen) {
+    if(!subject_message_id || unseen.length === 0) { return; }
+    $.ajax({
+      url: '/vest/talks/' + subject_message_id + '/seen',
+      method: 'PUT',
+      data: {'unseen': unseen.join(',')}
+      });
+  }
+
+
   /**
    * Prepares the HTML for the message and returns it.
    * @param msg {Object} The object containig the message
@@ -124,13 +173,16 @@
     template.attr('id', 'msg' + msg.id);
     //just to debug order by
     template.attr('title', 'msg' + msg.id);
-
+    if (msg.to_uid == user.id) { template.addClass('ui message'); }
     var date =
       typeof(msg.tstamp) === 'object' ?
       msg.tstamp :
       new Date(parseInt(msg.tstamp + '000')); //milliseconds
     template.find('.date').html(date.toLocaleString());
     template.find('.message').html(msg.message);
+    if(msg.seen && msg.to_uid != user.id){
+      template.find('.message').append(' <i title="displayed" style="color: #876A38" class="small checkmark outline icon"></i>');
+    }
     if (prev_msg.from_uid == msg.from_uid) {
       template.find('.from_uid_name').html('...');
     } else {
@@ -170,8 +222,9 @@
    * the element to which it is bound.
    * @return false
    */
-  function new_talk() {
+  function new_talk() {/* jshint validthis: true */
     $('#messages .ui.list p').remove(); //empty messages list
+
     set_talk_form({
       id: 0, // this will be the value of subject_message_id in the form
       subject: '',
@@ -185,6 +238,55 @@
     return false;
   } //end function new_talk()
 
+  /**
+   * Refresh the list of talks in the left sidebar
+   * with new unseen messages and the talks button
+   * with the sum of unseen messages. Prepend the
+   * list of talks with new talks.
+   * @param talks {Array} - list of talks
+   * @return {void}
+   */
+   function refresh_talks (talks) {
+      var talk_list = $('#talks ul ');
+      var unseen_sum = 0;
+      talk_list.find('li').remove();//empty talks list
+      for (var i = 0; i < talks.length; i++) {
+        var talk = talks[i];
+        var template = $($('#talk_item').html()); //copy
+        var a = template.find('a').first();
+        a.attr('id', 'u' + talk.id);
+        a.data(talk);
+        a.attr('href','/vest/messages/' + talk.id+'.json');
+        a.html(
+          (talk.from_uid == user.id? talk.to_uid_name: talk.from_uid_name) +
+          '(' + talk.subject + ')');
+        var b = template.find('b');
+        if(talk.unseen > 0 ){
+          b.html(talk.unseen);
+          unseen_sum = unseen_sum + talk.unseen;
+        }
+        else{
+          b.remove();
+        }
+        talk_list.append(template);
+      }//end for
+      var bt = $($('#talk_item').html()).find('b');
+      if (unseen_sum > 0) {
+        if($('#talks_button b').text().length){
+          $('#talks_button b').text(unseen_sum);
+        }
+        else {
+          $('#talks_button').append(bt.text(unseen_sum));
+        }
+      }
+      else {
+        $('#talks_button b').remove(); 
+      }
+    // re-bind onclick to filling-in messages in the #messages
+    $('#talks ul li a').click(get_messages);
+    return;
+  }
+
   // Functionality related to messages
   /**
    * Validates the message form and sends the message.
@@ -212,7 +314,7 @@
       post_message_success
     ).fail(function(data) {
       //TODO: replace the alert with a beautiful SemanticUI popup or box
-      alert('error:' + data.responseText);
+      window.alert('error:' + data.responseText);
       message_field.addClass('error');
 
     });
@@ -224,7 +326,7 @@
    * Executed upon succes of a sent message.
    * @param {Object} data the JSON returned by the server.
    */
-  function post_message_success(data) {
+  function post_message_success() {
     // Prepare the sent message to be appended to the list of messages.
     var form = $('#message_form').get(0);
     var msg = {
@@ -235,8 +337,8 @@
     $.each(form, function(i) {
       msg[form[i].name] = form[i].value;
     });
-    var last_i = VestTalk.json_messages.data.length - 1;
-    var prev_msg = VestTalk.json_messages.data[last_i] || {};
+    var last_i = VestTalk.messages.length - 1;
+    var prev_msg = VestTalk.messages[last_i] || {};
     var messages = $('#messages .ui.list');
     // fill in template and display the message
     fill_in_message_template(msg, prev_msg).appendTo(messages);
@@ -250,43 +352,61 @@
   /**
    * Gets new messages from a talk.
    * Similar to get_messages but only gets last 5 messages.
-   * @param {obj} form The form object from which we will get everything we need.
+   * Will not try to get new messages if the talk is brand new
+   * (i.e. has not started yet === is not saved in the database).
+   * @param {obj} form The form object from which we will get 
+   * the name-value pairs to be send.
+   * @param {int} limit  The number of messages to get - 5 by default.
    */
   function get_new_messages(form, limit) {
+    //console.log(form)
+    if(form.subject_message_id.value === 0) { return; }
+
+
     limit = limit ? limit : 5;
     var url = form.action + '/messages/' + form.subject_message_id.value +
       '.json?limit=' + limit;
-    $.get(url, append_messages_from_json, 'json');
+    $.get(url, function (json_data) {
+      append_messages_from_json(json_data);
+    } , 'json');
   }
 
   /**
-   * Gets new messages from a talk.
+   * Appends messages received from the server to the list on the screen.
    * Similar to list_messages_from_json but only appends messages to the messages box.
+   * Invokes set_seen() and refresh_talks().
    * @param {obj} form The form object from which we will get everything we need.
    */
-  function append_messages_from_json(new_json_messages) {
-    //remove the local message
+  function append_messages_from_json(JSON) {
+    //remove the local message if it exists
     $('#msg0').remove();
-    var js_messages = new_json_messages.data;
+    var js_messages = JSON.data;
     var messages = $('#messages .ui.list');
+    var unseen = [];
     for (var i in js_messages) {
       var msg = js_messages[i];
+      // collect yet unseen messages sent to the current user.
+      if(user.id === msg.to_uid && (msg.seen === null || msg.seen === 0)) { unseen.push(msg.id); }
       //skip the parent message
       if (msg.subject_message_id === 0) {
         continue;
       }
       //append the new message
       if (document.getElementById('msg' + msg.id) === null) {
-        var prev_msg = VestTalk.json_messages.data[VestTalk.json_messages.data.length -
+        var prev_msg = VestTalk.messages[VestTalk.messages.length -
           1];
         // fill in template and display the message
         fill_in_message_template(msg, prev_msg).appendTo(messages);
-        VestTalk.json_messages.data.push(msg);
+        VestTalk.messages.push(msg);
         // Scroll down to the last message
         messages.parent().scrollTop(messages.height());
       }
     } // end for( var i in...
+      set_seen((js_messages[1] ? js_messages[1].subject_message_id : 0 ), unseen);
+      refresh_talks(JSON.meta.talks);
+      //console.log(js_messages,unseen);
   } //end function append_messages_from_json(new_json_messages)
+
   /**
    * Finds new contacts for a user and displays them in div.results.
    * TODO: Replace this with Semantic UI Search module when API docs are ready.
@@ -306,12 +426,12 @@
           template.attr('data-name', results.data[i].name);
           template.find('i.icon').attr('data-id', results.data[i].id);
           template.find('span.label').text(results.data[i].name);
-          template.click(function(e){
+          template.click(function(){
             add_contact(template);
             return false;
           });
           $('#contacts .results').append(template);
-        };
+        }
     });
   }//end function find_contacts(e)
 
@@ -328,7 +448,7 @@
       user_item.data('href'), // /add_contact
       {id: user_item.data('id')},//form data
       // success
-      function add_contact_success (data, textStatus, xhr) {
+      function add_contact_success () {
         user_item.unbind('click'); //only once
         user_item.find('.comment.outline.icon').click(new_talk);
 
@@ -337,6 +457,6 @@
         $('#contacts ul.results').html('');//clean results
       }
     );
-    return false
+    return false;
   }
 })(jQuery); //execute
\ No newline at end of file
